package com.lockscreen.onelock

import android.app.admin.DevicePolicyManager
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.support.v7.app.AppCompatActivity
import android.widget.Toast
import com.dotools.umlibrary.UMPostUtils
import com.dotools.utils.DevicesUtils_HuaWei
import com.dotools.utils.DevicesUtils_Oppo
import com.tools.permissions.library.DOPermissions
import com.tools.permissions.library.Permissions


class OneLock : AppCompatActivity(), DOPermissions.DOPermissionsCallbacks {
    private val REQUEST_CODE = 11
    private val GAVE_REQUEST_CODE = 22
    private val REQUEST_CODETWO = 33
    private val REQUEST_CODETHREE = 44
    private val REQUEST_CODEFOUR = 55
    private var needOver = false
    private var componentName: ComponentName? = null
    private var manager: DevicePolicyManager? = null
    private val phoneVERSION = Build.VERSION.SDK_INT
    private var userOnResume = false


    override fun getComponentName(): ComponentName? {
        componentName = ComponentName(this, OneLockAdmin::class.java)
        return componentName
    }

    public override fun onCreate(paramBundle: Bundle?) {
        super.onCreate(paramBundle)
        getComponentName()
        manager = getSystemService(Context.DEVICE_POLICY_SERVICE) as DevicePolicyManager
        if (!SharedPreferences.getHasCreatedShortcut(this)) {
            createShortCut(this)
            SharedPreferences.setHasCreatedShortcut(this)
        }
        if (phoneVERSION >= 23) {
            if (!DOPermissions.getInstance().hasPermission(this, Permissions.READ_EXTERNAL_STORAGE, Permissions.WRITE_EXTERNAL_STORAGE, Permissions.READ_PHONE_STATE)) {
                DOPermissions.getInstance().getPermissions(this, "运行程序需要权限", 111, Permissions.READ_EXTERNAL_STORAGE, Permissions.WRITE_EXTERNAL_STORAGE, Permissions.READ_PHONE_STATE)
            } else {
                start()
            }
        } else {
            start()
        }
    }


    private fun start() {
        if (phoneVERSION >= 21 && SharedPreferences.isShowedPermission(applicationContext) != 1) {//显示授权页
            startActivityForResult(Intent(this@OneLock, ClearPreActivity::class.java), REQUEST_CODEFOUR)
        } else {
            lock()
        }
    }


    private fun lock() {
        if (checkHuaWei() || checkOppo()) {
            needOver = true
            startActivity(Intent(this, NewLockActivity::class.java))
            this.overridePendingTransition(0, 0)
        } else {
            adminLock()
        }
    }

    private fun checkHuaWei(): Boolean {
        return (DevicesUtils_HuaWei.isHuaWeiRom() && phoneVERSION >= 26 && phoneVERSION < 29)
    }

    private fun checkOppo(): Boolean {
        val version = DevicesUtils_Oppo.getOPPORomVersionCode()
        return if (version != null) {
            (DevicesUtils_Oppo.isOppoDevice() && (version == "V5" || version == "V6"))
        } else {
            false
        }
    }

    private fun adminLock() {
        if (manager!!.isAdminActive(componentName!!)) {
            manager!!.lockNow()
            needOver = true
        } else {
            val intent = Intent(this, JoinActivity::class.java)
            startActivityForResult(intent, GAVE_REQUEST_CODE)
        }
    }


    override fun onPermissionsDenied(requestCode: Int, perms: MutableList<String>) {
        start()
    }

    override fun onPermissionsGranted(requestCode: Int, perms: MutableList<String>) {
        start()
    }

    override fun onRequestPermissionsResult(requestCode: Int, permissions: Array<out String>, grantResults: IntArray) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        DOPermissions.getInstance().setRequestPermission(this, requestCode, permissions, grantResults)
    }

    override fun onResume() {
        super.onResume()
        userOnResume = true
        UMPostUtils.onActivityResume(this)
        UMPostUtils.onEvent(applicationContext, "click")
        if (needOver) {
            finish()
        }
    }

//    private fun checkClearMemory() {
//        if (SharedPreferences.HasPermission(applicationContext)) {
//            val cleanUtils = CleanUtils(applicationContext)
//            cleanUtils.ClearMemory(applicationContext)
//        }
//    }


    override fun onPause() {
        super.onPause()
        UMPostUtils.onActivityPause(this)
//        checkClearMemory()
    }


    override fun onDestroy() {
        super.onDestroy()
        if (!userOnResume) {
            UMPostUtils.onActivityPause(this)
            userOnResume = false
        }
    }

    private fun createShortCut(context: Context) {
        // 创建快捷方式的Intent
        val shortcutintent = Intent("com.android.launcher.action.INSTALL_SHORTCUT")
        // 不允许重复创建
        shortcutintent.putExtra("duplicate", false)
        // 需要现实的名称
        if (BuildConfig.IS_APPNAME_DD) {
            shortcutintent.putExtra(Intent.EXTRA_SHORTCUT_NAME, context.getString(R.string.app_name_dd))
        } else {
            shortcutintent.putExtra(Intent.EXTRA_SHORTCUT_NAME, context.getString(R.string.app_name))
        }
        // 快捷图片
        val icon = Intent.ShortcutIconResource.fromContext(context, R.drawable.ic_launcher)
        shortcutintent.putExtra(Intent.EXTRA_SHORTCUT_ICON_RESOURCE, icon)
        // 点击快捷图片，运行的程序主入口
        shortcutintent.putExtra(Intent.EXTRA_SHORTCUT_INTENT, Intent(context, OneLock::class.java))
        // 发送广播。OK
        context.sendBroadcast(shortcutintent)
    }


    override fun onActivityResult(paramInt1: Int, paramInt2: Int, paramIntent: Intent?) {
        super.onActivityResult(paramInt1, paramInt2, paramIntent)
        if (paramInt1 == REQUEST_CODE) {
            if (manager!!.isAdminActive(componentName!!)) {
                Toast.makeText(this, getString(R.string.activated), Toast.LENGTH_SHORT).show()
            }
            finish()
        } else if (paramInt1 == GAVE_REQUEST_CODE) {
            if (paramIntent != null) {
                val istrue = paramIntent.getBooleanExtra("isFromCover", false)
                if (istrue) {
                    val str = getString(R.string.lock_screen_tips)
                    val localIntent = Intent("android.app.action.ADD_DEVICE_ADMIN")
                    localIntent.putExtra("android.app.extra.DEVICE_ADMIN", componentName)
                    localIntent.putExtra("android.app.extra.ADD_EXPLANATION", str)
                    startActivityForResult(localIntent, REQUEST_CODE)
                } else {
                    finish()
                }
            }
        } else if (paramInt1 == REQUEST_CODETWO) {
            if (manager!!.isAdminActive(componentName!!)) {
                finish()
            } else {
                val intent = Intent(this, JoinActivity::class.java)
                startActivityForResult(intent, GAVE_REQUEST_CODE)
            }
        } else if (paramInt1 == REQUEST_CODETHREE) {
            if (phoneVERSION >= 21 && SharedPreferences.isShowedPermission(applicationContext) != 1) {
                startActivityForResult(Intent(this@OneLock, ClearPreActivity::class.java), REQUEST_CODEFOUR)
            } else {
                lock()
            }
        } else if (paramInt1 == REQUEST_CODEFOUR) {
            lock()
        }
    }
}
